/**
 * Program POSIXové Fronty
 * @autor František Kučera <franta-hg@frantovo.cz>
 */

#include <program.h>
#include <stdlib.h>
#include <iostream>
#include <mqueue.h>
#include <readline/readline.h>
#include <signal.h>

using namespace std;

/**
 * Spuštění programu „POSIXovéFronty“
 */
int main() {

    cout << "Vítejte v programu POSIXovéFronty" << endl;

    /** Nastavíme reakci na signál SIGTERM */
    signal(SIGTERM, zpracujSignal);
    signal(SIGINT, zpracujSignal);

    /** Nastavíme vlastnosti fronty */
    mq_vlastnosti.mq_maxmsg = POCET_ZPRAV_VE_FRONTE;
    mq_vlastnosti.mq_msgsize = DELKA_ZPRAVY;

    /** Program pracuje buď v režimu přijímání nebo odesílání */
    if (isOdesilaciRezim()) {
        odesilej();
    } else {
        prijimej();
    }

    /** Sem se nedostaneme – program končí, až když dostane patřičný signál. */
    return (EXIT_SUCCESS);
}

bool isOdesilaciRezim() {

    cout << "Zvolte režim: (p)řijímací / (o)desílací" << endl;

    char odpoved = cin.get();
    cin.get();

    if (odpoved == 'o') {
        return true;
    } else if (odpoved == 'p') {
        return false;
    } else {
        return isOdesilaciRezim();
    }
}

void odesilej() {
    cout << "Odesíláme…" << endl;

    mqd_t fronta = mq_open(NAZEV_FRONTY, O_RDWR | O_CREAT, S_IRWXU | S_IRWXG, &mq_vlastnosti);

    char zprava[] = "zpráva";

    int vysledek = mq_send(fronta, zprava, strlen(zprava) + 1, 0);
    if (vysledek == 0) {
        cout << "Zpráva odeslána." << endl;
    } else {
        cout << "Při odesílání zprávy došlo k chybě." << endl;
    }

    mq_close(fronta);
}

void prijimej() {
    cout << "Přijímáme…" << endl;

    mqd_t fronta = mq_open(NAZEV_FRONTY, O_RDONLY);
    char s[DELKA_ZPRAVY + 1];
    s[DELKA_ZPRAVY] = '\0';
    if (mq_receive(fronta, s, DELKA_ZPRAVY, NULL) > 0) {
        printf("Přijatá zpráva: %s\n", s);
    } else {
        cout << "Nepodařilo se přijmout zprávu." << endl;
    }

    mq_close(fronta);
}

void zpracujSignal(int signal) {
    if (signal == SIGTERM || signal == SIGINT) {
        cout << "Končím program na základě signálu " << signal << "." << endl;
        exit(1);
    }
}

